/*====================================================================================
This source file is part of the TechTools Plugin Development Kit. 
Copyright (c) 2011 by TechTools  

DISCLAIMERS:
 - NO WARRANTIES
     This software is provided 'As Is', without any express or implied
     warranty of any kind, including but not limited to any warranties 
     of merchantability, noninfringement, or fitness of a particular 
     purpose. The Copyright holders do not warrant or assume responsibility
     for the accuracy or completeness of any information contained within
     this software. 
 - LIMITATION OF LIABILITY
     In no event shall the copyright holders be liable for any damages 
     (including, without limitation, lost profits, business interruption, 
     or lost information) rising out of use of or inability to use this 
     software, even if advised of the possibility of such damages.  In no
     event will the copyright holders be liable for loss of data or for 
     indirect, special, incidental, consequential (including lost profit),
     or other damages based in contract, tort or otherwise. The copyright
     holders shall have no liability with respect to the content of the 
     software or any part thereof, including but not limited to errors or 
     omissions contained therein, libel, infringements of rights of publicity,
     privacy, trademark rights, business interruption, personal injury, 
     loss of privacy, moral rights or the disclosure of confidential
     information.  

Use and redistribution of this software or of derived works, in source or
compiled form is permitted as long as the following restrictions are observed:

- The above copyright(s), disclaimers and these restrictions are retained in the
  source files and, if distributed in compiled form, must be duplicated in 
  documentation or other materials and provided with the distribution.
- The derived work is used exclusively as a plugin to the TechTools DigiView 
  software, to process data captured with TechTools hardware.
- The copyright holders' names may not be used to endorse or to promote any 
  product or derived works.

Any other use is prohibited without express, written permission from TechTools.

email: support@tech-tools.com,  sales@tech-tools.com 
web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

// Reproduces the built-in I2C post processor.
// Demonstrates most of the plugin features:
//  adding simple user options, formatting, framing, lookup tables

#include "../plugin.h"

 // indexes into our field format definitions
static enum fieldnames {TfSTART,TfSTOP,TfADDR,TfADDR8,TfDATA,TfACK,TfNAK,TfWRITE,TfREAD,TfALLCALL,
                        TfSTARTBYTE,TfHSMASTER,TfCBUS,TfRESERVED,Tf10bit,TfTRUNC,FNEND};
 
 // indexes into our user configuration options
static enum cfgopts {TcfgAcks,Tcfg8bit,TcfgIdles,TcfgEND};

 // EVENTS from the I2C pre-parser
static enum  TEVENTS {TEStart,TEStartByte,TEAddr,TEAllcall, TECbus,TEHSMast,TEReserved,
	                  TE10bit,TEDir,TEAck,TEData,TEStop,TETrunc,TERestart,TEFieldIdle};

static bool showidles;     // user option to show idle periods
static bool show8bit;      // user option to interpret addesses as 8 bit
static bool showacks;      // user option to show/hide ACKs

// unused stubs
void OnLoad() {}       // no one-time initialization needed
void OnUnLoad() {}     // no final cleanup needed
void SetInitItem(unsigned char ID, unsigned char subID, int value) {} // don't care about globals
void StartOfData() {}  // nothing to init at start of data
void EndOfData() {}    // no end-of-parsing finalization needed

// configuration string lists
void GetStrList(int ID, vector<string> &strl)
{	switch (ID) 
	{ 
	case 0:  // Plugin description
		strl.push_back("I2C-Base plugin example V1.0"); 
		break; 

	case 1:  // user options
		strl.assign(TcfgEND,""); // allocate and init the list
		strl[TcfgAcks]  = "Show ACKs,checkbox,false";
		strl[Tcfg8bit]  = "Show Addresses as,radio,0,Decoded 7bit,Raw 8bit (Addr+Dir)";
		strl[TcfgIdles] = "Show IDLEs,checkbox,false";
		break;

	case 2:  // Field formats
		strl.assign(FNEND,"");  // allocate and init the list
		strl[TfSTART]    = "START,Green,BLACK,{0%0:0}";  // uses LSB to print 'S' for start or 'Sr' for restart
		strl[TfSTOP]	 = "STOP,YELLOW,BLACK,P";
		strl[TfADDR]	 = "Addr,Green,BLACK,{7:1}";             // prints top 7 bits as 7bit Address
		strl[TfADDR8]    = "Addr8,Green,BLACK,{7:0} ({1%0:0})";  // prints 8bit addr, uses LSB as lookup for (R or W)
        strl[TfDATA]	 = "Data,,,{7:0}";
		strl[TfACK]		 = "Ack,C0C0C0,BLACK,A";    // defined separate Ack/Nak formats so we could have different
		strl[TfNAK]		 = "Nak,YELLOW,BLACK,N";    //    colors. (instead of 1 format with lookup table)
		strl[TfWRITE]	 = "WRITE,BLUE,WHITE,W";    // separate Read/Write formats as well
		strl[TfREAD]	 = "READ,006000,WHITE,R";
		strl[TfALLCALL]	 = "General-CALL,Green,Black,Gen-Call";
		strl[TfSTARTBYTE]= "START-BYTE,Green,Black,START";
		strl[TfHSMASTER] = "HS-Master,Green,Black,HSMaster:{2:0}";
		strl[TfCBUS]	 = "CBUS,C0C0C0,Black,CBUS (ignored until next START)";
		strl[TfRESERVED] = "Reserved,Green,Black,Reserved:{7:1}";
		strl[Tf10bit]	 = "10bitMode,Green,Black,10bitMode:{2:1}";
		strl[TfTRUNC]	 = "Truncated,E0E0E0,Black,T:{7:1}";
		break;

	case 3:  // pre-parser
		strl.push_back("I2C"); 
		break;    

	case 4:  // Framework version
		strl.push_back("1");  // require framework version 1 or above	
		break;    

	case 5:  // lookup tables
		strl.push_back("0,0,S");  // a 2-entry table
		strl.push_back("0,1,Sr");
		strl.push_back("1,0,W");  // another 2-entry table
		strl.push_back("1,1,R");
	}
}


void SetCfgItem(unsigned char ID, unsigned char subID, int value)
{  // set the user configured options we provided
	switch(ID)
	{   
		case TcfgAcks:  if (subID == 0) showacks  = (value != 0); break;
		case Tcfg8bit:  if (subID == 0) show8bit  = (value != 0); break;
		case TcfgIdles: if (subID == 0) showidles = (value != 0); break;
	}
} 


// This is the actual post-processor.  The pre-parser does most of the work.  We mostly just define
// the display format.  We added a couple of user options to control the first byte (address/direction)
// interpretation and the visibility of ACKs.  We also let the user decide if IDLEs should be shown.
void Parse(int64 timestamp, Data64 data)
{ 
	switch(data.bytes[6])  // the I2C event code is in byte 6
	{
	case TEStart:      // Start condition detected     
		data.bytes[0] = 0;  // print 'S'
		StartFrame(timestamp,data,TfSTART); 
		if (showidles) EndField(timestamp);
		break;

	case (TERestart):  // Start without a preceeding stop condition
		data.bytes[0] = 1;  // print 'Sr'
		StartFrame(timestamp,data,TfSTART); 
		if (showidles) EndField(timestamp);
		break;

	case (TEHSMast):  // High Speed Master Code
		StartField(timestamp,data,TfHSMASTER); 
		break;

	case (TE10bit):   // 10bit Address mode
		StartField(timestamp,data,Tf10bit); 
		break;
	
	case (TEAddr):    // normal Address
		if (show8bit) StartField(timestamp,data,TfADDR8); 
		else StartField(timestamp,data,TfADDR); 
		break;

    case (TEAck):      // ACK/NAK field event
		if ((data.bytes[0] & 1) == 1)  StartField(timestamp,data,TfNAK);
		else if (showacks)         StartField(timestamp,data,TfACK);

		if (showidles)
			EndField(timestamp);
		break;
    
	case (TEDir):    // Direction Event.  LSB holds the direction (0=>Wt)
		if (!show8bit)  // we don't show the R/W field in 8bit address mode
		{
			if ((data.bytes[0] & 0x01) == 0)	StartField(timestamp,data,TfWRITE);
		    else      					    StartField(timestamp,data,TfREAD);
		    if (showidles) 			        EndField(timestamp);
		}
		break;
    
	case (TEReserved):	// Reserved device address
		StartField(timestamp,data,TfRESERVED); 
		break;

    case (TEData):   // normal data field		
		StartField(timestamp,data,TfDATA); 
		break;

	case (TEStop):		// Stop condition detected
		StartField(timestamp,data,TfSTOP); 
		EndField(timestamp);
		break;

	case (TETrunc):		// partial field
		StartField(timestamp,data,TfTRUNC); 
		break;

    case (TEFieldIdle):	// termination time for current field
		if (showidles) EndField(timestamp); 
		break;
	}
}
