/*====================================================================================
This source file is part of the TechTools Plugin Development Kit. 
Copyright (c) 2011 by TechTools  

DISCLAIMERS:
 - NO WARRANTIES
     This software is provided 'As Is', without any express or implied
     warranty of any kind, including but not limited to any warranties 
     of merchantability, noninfringement, or fitness of a particular 
     purpose. The Copyright holders do not warrant or assume responsibility
     for the accuracy or completeness of any information contained within
     this software. 
 - LIMITATION OF LIABILITY
     In no event shall the copyright holders be liable for any damages 
     (including, without limitation, lost profits, business interruption, 
     or lost information) rising out of use of or inability to use this 
     software, even if advised of the possibility of such damages.  In no
     event will the copyright holders be liable for loss of data or for 
     indirect, special, incidental, consequential (including lost profit),
     or other damages based in contract, tort or otherwise. The copyright
     holders shall have no liability with respect to the content of the 
     software or any part thereof, including but not limited to errors or 
     omissions contained therein, libel, infringements of rights of publicity,
     privacy, trademark rights, business interruption, personal injury, 
     loss of privacy, moral rights or the disclosure of confidential
     information.  

Use and redistribution of this software or of derived works, in source or
compiled form is permitted as long as the following restrictions are observed:

- The above copyright(s), disclaimers and these restrictions are retained in the
  source files and, if distributed in compiled form, must be duplicated in 
  documentation or other materials and provided with the distribution.
- The derived work is used exclusively as a plugin to the TechTools DigiView 
  software, to process data captured with TechTools hardware.
- The copyright holders' names may not be used to endorse or to promote any 
  product or derived works.

Any other use is prohibited without express, written permission from TechTools.

email: support@tech-tools.com,  sales@tech-tools.com 
web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

// RawState.cpp
// This is a simple example of a full plugin. (Uses the built-in RAW parser as a
// preparser.)  It demonstrates working at the raw data level: exracting channel data,
// detecting edges, generating fields and frames, and interacting with user settings.

#include "../plugin.h"

// A little short-hand... These give readable names to the Configuration Options and
// field formats described below.  The final entries are used as sentinals to mark the length/ends  

//  configuration items (case 1)
static enum cfgnames { CLK_MASK,MYDATA_MASK,EDGE_SEL,EN_MASK,
	EN_LEVEL,STATES_PER_FRAME,SKIP,SYNC_DISABLED,CFGEND};

// field formats (case 2)
static enum fieldnames { F_1, F_2 };

// unused stubs
void OnLoad() {}
void SetInitItem(unsigned char ID, unsigned char subID, int value) {}
void OnUnLoad() {}
void EndOfData() {} // we do not need to do any clean up.


// called to retrieve descriptions and configuration information from the plug-in
// fills the provided vector container with the requested set of strings
void GetStrList(int ID, vector<string> &strl)
{
	// You are free to store your strings any way you wish as long as you
	// copy them into the provided vector when requested. You might choose
	// to store them as constant arrays or pre-initialized vectors.  We chose
	//to load them directly in-line here, keeping all configuration in one place.
	switch (ID)
	{ 
	case 0:   // Plug-in Description
		// These lines are displayed in the signal editor for identification
		// but serve no other purpose. For best results, keep to 4 or fewer lines.
		strl.push_back(	"Example Raw State plugin V1.0");
		break;

	case 1:   // configuration options
		// These are the items displayed in the SIGNAL EDITOR. They allow the
		// user to configure the plug-in for a given signal. You must provide
		// at least 1 chanselect type option to allow the user to assign physical
		// channel(s) to the signal. 
		strl.assign(CFGEND,"");  // allocate and init the list
		strl[CLK_MASK]         =  "Clock Channel,chanselect,1,1,1,false,false";
		strl[MYDATA_MASK]      =  "Data Channels,chanselect,2,1,36,true,false";
		strl[EDGE_SEL]         =  "Clock On,combo,1,Rising Edge,Falling Edge,Both Edges";
		strl[EN_MASK]          =  "Enable Channel,chanselect,4,1,1,false,true";
		strl[EN_LEVEL]         =  "Enable Level,combo,0,Low,High";
		strl[STATES_PER_FRAME] =  "States per Frame,edit,4";
		strl[SKIP]             =  "Skip States,spinner,0,0,255,1";
		strl[SYNC_DISABLED]    =  "Sync on Disable,checkbox,0";
		break;

	case 2:  // Field Formats
		// These tell the app how to display your fields. You must define at least 
		// one field format.
		strl.push_back(	"First Field,BLUE,WHITE,{}");
		strl.push_back(	"Other,c0c0c0,BLACK,{}");
		break;

	case 3:  //return which preparser we need
		strl.push_back("RAW");
		break;

	case 4: // return minimim framework ver we need
		strl.push_back("1");
		break;
	}
}


static bool first_sample;
static uint64 BusHighBit;
static uint64 BusLowBit;

// our configuration items
static Data64 datamask;
static bool InvertFlag;
static Data64 clkmask;
static bool either_edge;
static bool clkedge;
static Data64 enmask;
static bool always_en;
static bool en_level;
static int states_per_frame;
static int skip_count;
static bool sync_on_disable;


// called to set our configuration options.
// These represent the choices the user made in response to your Configuration Options
// You can store and retrieve your configuration items any way you want.
// We chose to store them directly in named variables.
// The ID corresponds to the index of the Configuration Options item.
// eg 0 => the first configuration string you provided....

void SetCfgItem(unsigned char ID, unsigned char subID, int value)
{
	switch (ID)
	{
	case MYDATA_MASK:  // data bus mask
		switch (subID)
		{
		case 0: InvertFlag = (value & 0x01); break;
		case 1: datamask.lowint = value;   break;
		case 2: datamask.highint = value;  break;
		}
		break;

	case CLK_MASK: 
		switch (subID)
		{
		case 1: clkmask.lowint = value;  break;
		case 2: clkmask.highint = value; break;
		}
		break;

	case EDGE_SEL:
		if (subID == 0)
		{
			if (value == 0) {either_edge = false; clkedge = true;}  // rising edge
			else if (value == 1) {either_edge = false; clkedge = false;} // falling edge
			else  {either_edge = true; clkedge = false;}  // either edge
		}
		break;

	case EN_MASK: 
		switch (subID)
		{
		case 0: always_en = ((value & 2) == 2); break;  // ignore enable if en_mask is disabled
		case 1: enmask.lowint = value;  break;
		case 2: enmask.highint = value; break;
		}
		break;

	case EN_LEVEL:
		if (subID == 0) en_level = (value == 1);
		break;

	case STATES_PER_FRAME:
		if (subID == 0) states_per_frame = int(value);
		break;

	case SKIP:
		if (subID == 0) skip_count = int(value);
		break;

	case SYNC_DISABLED:
		if (subID == 0) sync_on_disable = (value != 0);
		break;
	// ignore illegal cfg items
	}
}


// Called immediately after setting configuration options and before data
// streaming begins. Do any initialization you require here. In particular,
// note that you should assume that the configuration options have changed.
void StartOfData()
{	// some of our initialization is dependent on SEEing the first sample...
	// do it in the first call to 'parse'
	first_sample = true;

	// Pre-calculate the bus-type signal's upper and lower channel usage.  This greatly
	// speeds up the data extraction process during data streaming ('parse' calls).
	FindChannelLimits(datamask.i64,BusHighBit,BusLowBit);
}


// Called when a new data sample has been received.
// This is the protocol parser itself.  It must examine each data sample as it
// arrives, send back field information when appropriate for this protocol and
// return immediately.  It must never loop or attempt to use stdio.  It must 
// maintain its state between calls through statics.
// NOTE that this could get called on the order of 250,000 times per capture per signal,
// so efficiency is important.  We pre-converted many of the configurtion items to 
// bools in the setcfg routine to minimize masking, shifting and int64 compares here.
void Parse(int64 timestamp, Data64 rawdata)
{
    static int FieldNum = 0;  // tracks states per frame
    static bool clk_was;   // remembers clk level for edge detection
    static bool en_was;    // remembers enabled state for edge detection
	
	bool clk_bit;
	bool enabled;
	Data64 mydata;

	// extract our control signals (defer extracting the data signal until/unless needed.
	// NOTE: the PACK operation is expensive (even with pre-calculated limits).  It should only
	// be used with bus type signals (> 1 bit) and only when needed.  For boolean signals, 
	// it is far more efficient to just mask directly:
	clk_bit = ((rawdata.i64 & clkmask.i64) != 0);
	enabled = ((always_en) | (en_level == ((rawdata.i64 & enmask.i64) != 0)));

	// if this is the first sample, establish a baseline by noting the current
	// control signal levels and then return.
	if (first_sample)
	{ 
		first_sample = false;  // only one 'first sample'
	    FieldNum =  (sync_on_disable & !enabled)?0:-skip_count;
		clk_was = clk_bit;  // note current clock level
		en_was = enabled;   // and enabled state
		return;
	}

	// first see if we want this sample
	if ((enabled)                                   // enable is active
		&  (clk_bit != clk_was)                    // & the clock transitioned
		&  (either_edge | (clkedge == clk_bit)))    // & it is a clk edge we want
	{  
		if (FieldNum < 0) 
		  FieldNum += 1;
		else
		{
		// extract the state bus from the rawdata
		if (InvertFlag )  	// invert the data if the user configured us to do so
			mydata.i64 = pack(~rawdata.i64, datamask.i64,BusHighBit,BusLowBit);
		else
			mydata.i64 = pack(rawdata.i64, datamask.i64,BusHighBit,BusLowBit);

		// select framing and field formatting, based on which field we are dealing with.
		// This example does not look at higher levels of the protocol to determine framing.
		// The user simply tells us how many states to put in each frame.  They can visually
		// sync a frame by changing the 'skip' cfg.  The user can also tell us to re-sync
		// on every disable.  
		
		// If the user specified 1 or fewer state per frame, it implies NO FRAMING
        if (states_per_frame <= 1)
			StartField(timestamp, mydata, F_1);  // use format definition 1, no frame
		else
		{
		  if (FieldNum == 0)   // start a new frame&field and use field ID 1 for formatting
			StartFrame(timestamp, mydata, F_1);
		  else  // start a new field but not frame. Use format specifier 2 for variety
			StartField(timestamp, mydata, F_2); 

		  FieldNum = (FieldNum + 1) % states_per_frame; 
		}
		}
	}

	else if (sync_on_disable & en_was & !enabled)  // if resync and we see inactive edge of enable.
	{    // terminate any existing frame/field (note data and formatID are ignored..
		EndFrame(timestamp); //terminate current frame and start idle
		FieldNum = 0;  // reset so next valid sample starts a new frame
	}

      // remember current control states for future edge detection
	clk_was = clk_bit; 
	en_was = enabled; 
}

